@@ -191,7 +191,9 @@ QStringList qc_splitflags(const QString &flags)
 	bool escaped = false;
 	QChar quote, backslash = QLatin1Char('\\');
 	QString buf;
+#ifdef PATH_MAX
 	buf.reserve(PATH_MAX);
+#endif
 	for (int i=0; i < flags.length(); i++) {
 		if (searchStart && flags[i].isSpace()) {
 			continue;
@@ -429,6 +431,10 @@ public:
 			conf->addLib(libs);
 		//if(!other.isEmpty())
 		//	conf->addExtra(QString("QMAKE_CFLAGS += %1\n").arg(other));
+
+		if(!required)
+			conf->addDefine("HAVE_PKG_" + qc_escapeArg(pkgname).toUpper());
+
 		return true;
 	}
 };
