@@ -12,7 +12,8 @@ preserved.
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
-#include <unistd.h>
+//#include <unistd.h>
+#include <limits.h>
 #include "embed.h"
 
 #if defined(WIN32) || defined(_WIN32)
@@ -25,12 +26,24 @@ preserved.
 
 #ifdef QC_OS_WIN
 static char *qconftemp_path = "qconftemp";
+static char path_separator = ';';
 #else
 static char *qconftemp_path = ".qconftemp";
+static char path_separator = ':';
 #endif
 
 static int qc_verbose = 0;
-static char *ex_qtdir = 0;
+static char *ex_qtdir = NULL;
+static char *qc_qtselect = NULL;
+static char *qtsearchtext="4 or 5";
+
+static char *prefix = NULL;
+static char *bindir = NULL;
+static char *includedir = NULL;
+static char *libdir = NULL;
+static char *datadir = NULL;
+
+static int run_buffer_stdout(const char *command, char *out_buf, size_t buf_size);
 
 enum ArgType
 {
@@ -340,12 +353,15 @@ static char *check_qmake_path(const char *qtdir)
 {
 	char *str;
 
-	str = strdup(qtdir);
+	str = separators_to_native(qtdir);
 #ifdef QC_OS_WIN
-	str = append_free(str, "/bin/qmake.exe");
+	str = append_free(str, "\\qmake.exe");
 #else
-	str = append_free(str, "/bin/qmake");
+	str = append_free(str, "/qmake");
 #endif
+	if (qc_verbose) {
+		printf("Check if \"%s\" exists\n", str);
+	}
 	if(file_exists(str))
 	{
 		return str;
@@ -357,17 +373,114 @@ static char *check_qmake_path(const char *qtdir)
 	}
 }
 
+static int qmake_query(const char *qmake_path, const char *var, char *out_buf, size_t buf_size)
+{
+	char command[PATH_MAX];
+	int cnt = snprintf(command, sizeof(command) - 1, "%s -query %s", qmake_path, var);
+	if (cnt > 0 && cnt < (int)sizeof(command))
+	{
+		command[sizeof(command) - 1] = '\0'; // To be sure line has null-terminator
+		return run_buffer_stdout(command, out_buf, buf_size);
+	}
+	return 0; // fail
+}
+
+static int qmake_query_maj_ver(const char *qmake_path, char *out_buf, size_t buf_size)
+{
+	if (qmake_query(qmake_path, "QT_VERSION", out_buf, buf_size)) {
+		int at = index_of(out_buf, '.');
+		if (at > 0)
+		{
+			out_buf[at] = '\0';
+			return 1;
+		}
+	}
+	return 0;
+}
+
+// modifies contents of spec_path and returns pointer to spec name
+char *extract_spec_name(char *spec_path)
+{
+	char *end = spec_path;
+	while (!iscntrl(*end)) {
+		if (*end == '/') {
+			if (iscntrl(*(end+1))) { // next is end of line
+				break;
+			} else {
+				spec_path = end + 1;
+				end = spec_path + 1;
+			}
+		} else
+			end++;
+	}
+	*end = 0;
+	return spec_path;
+}
+
+int check_qtversion(char *path, char *version)
+{
+	int res = 0;
+
+	if (version)
+	{
+		char buf[20]; // version string output always small
+		if (qmake_query_maj_ver(path, buf, sizeof(buf))) {
+			res = !strcmp(version, buf);
+			if (!res && qc_verbose)
+			{
+				printf("Warning: %s not for Qt %s\n", path, qtsearchtext);
+			}
+		}
+	}
+	else
+	{
+		res = 1;
+	}
+
+	return res;
+}
+
+static char* parse_qtselect(char *src)
+{
+	unsigned long int ver;
+	if (src) {
+		if (src[0] == 'q' && src[1] == 't')
+			src += 2;
+
+		ver = strtoul(src, NULL, 10);
+		if (ver > 0 && ver < ULONG_MAX) {
+			qc_qtselect = strdup(src);
+			qtsearchtext = qc_qtselect;
+			return qc_qtselect;
+		}
+	}
+	return NULL;
+}
+
 static char *find_qmake()
 {
 	char *qtdir;
 	char *path;
+	char try_syspath = 1;
+
+	if (!qc_qtselect)
+		parse_qtselect(get_envvar("QT_SELECT"));
+
 
 	qtdir = ex_qtdir;
 	if(qtdir)
 	{
+		qtdir = append_free(strdup(qtdir), "\\bin");
 		path = check_qmake_path(qtdir);
+		free(qtdir);
+
 		if(path)
+		{
+			if (!check_qtversion(path, qc_qtselect))
+				return NULL;
 			return path;
+		}
+		try_syspath = 0;
 	}
 	if(qc_verbose)
 		printf("Warning: qmake not found via --qtdir\n");
@@ -375,16 +488,73 @@ static char *find_qmake()
 	qtdir = get_envvar("QTDIR");
 	if(qtdir)
 	{
+		qtdir = append_free(strdup(qtdir), "\\bin");
 		path = check_qmake_path(qtdir);
+		free(qtdir);
+
 		if(path)
+		{
+			if (!check_qtversion(path, qc_qtselect))
+				return NULL;
 			return path;
+		}
+		try_syspath = 0;
 	}
 	if(qc_verbose)
 		printf("Warning: qmake not found via %%QTDIR%%\n");
 
+	/* if not set explicitly try something implicit */
+	if (try_syspath) {
+		char *paths = strdup(get_envvar("PATH"));
+		qtdir = paths;
+		while (1)
+		{
+			int at = index_of(qtdir, path_separator);
+			if (at > 0)
+				qtdir[at] = '\0';
+			path = check_qmake_path(qtdir);
+			if (path && check_qtversion(path, qc_qtselect))
+			{
+				free(paths);
+				return path;
+			}
+
+			if (at > 0)
+				qtdir += at + 1;
+			else
+				break;
+		}
+		free(paths);
+	}
+
+	if(qc_verbose)
+		printf("Warning: qmake not found in PATH\n");
+
 	return NULL;
 }
 
+static int run_buffer_stdout(const char *command, char *out_buf, size_t buf_size)
+{
+	FILE *file;
+
+	file = popen(command, "r");
+	if (file)
+	{
+		size_t bytes_left = buf_size - 1; // 1 for null terminator
+		size_t bytes_read;
+		while (bytes_left && (bytes_read = fread(out_buf, 1, bytes_left, file)) > 0)
+			bytes_left -= bytes_read;
+
+		out_buf[buf_size - bytes_left - 1] = '\0';
+		pclose(file);
+
+		return 1; // ok
+	}
+
+	out_buf[0] = 0; // just in case
+	return 0; // fail
+}
+
 static int run_silent_stdout(const char *cmd)
 {
 	char *str;
@@ -430,6 +600,8 @@ static int run_conflog_all(const char *cmd)
 #else
 	str = append_free(str, " >../conf.log 2>&1");
 #endif
+	if (qc_verbose)
+		printf("Starting \"%s\"\n", str);
 	ret = system(str);
 	free(str);
 
@@ -478,13 +650,13 @@ static int qc_removedir(const char *path)
 
 #ifdef QC_OS_WIN
 	str = strdup("deltree /y ");
-	str = append_free(str, qconftemp_path);
+	str = append_free(str, path);
 	ret = run_silent_all(str);
 	free(str);
 	if(ret != 0)
 	{
 		str = strdup("rmdir /s /q ");
-		str = append_free(str, qconftemp_path);
+		str = append_free(str, path);
 		ret = run_silent_all(str);
 		free(str);
 	}
@@ -551,31 +723,50 @@ static int try_make(const char *makecmd, char **maketool)
 
 	str = strdup(makecmd);
 	str = append_free(str, " clean");
-	ret = run_silent_all(str);
+	ret = run_conflog_all(str);
 	free(str);
 	if(ret != 0)
 		return 0;
 
-	if(run_conflog_all(makecmd) != 0)
+	if(run_conflog_all(makecmd) != 0) {
+		if (qc_verbose)
+			printf("\"%s\" failed\n", makecmd);
 		return 0;
+	}
 
 	*maketool = strdup(makecmd);
 	return 1;
 }
 
-static char *maketool_list[] =
+static char *maketool_list_common[] =
 {
-	"make",
 	"mingw32-make",
+	"make",
+	"jom",
 	"nmake",
 	NULL
 };
 
-static int do_conf_create(qcdata_t *q, const char *qmake_path, char **maketool)
+static char *maketool_list_mingw[] =
+{
+	"make",
+	"mingw32-make",
+	NULL
+};
+
+static char *maketool_list_vs[] =
+{
+	"jom",
+	"nmake",
+	NULL
+};
+
+static int do_conf_create(qcdata_t *q, const char *qmake_path, const char *spec, char **maketool)
 {
 	char *str;
 	int n;
 	int at;
+	char **maketool_list;
 
 	if(!qc_ensuredir(qconftemp_path))
 		return 0;
@@ -598,6 +789,13 @@ static int do_conf_create(qcdata_t *q, const char *qmake_path, char **maketool)
 	free(str);
 
 	at = -1;
+	if (strstr(spec, "win32-msvc")) {
+		maketool_list = maketool_list_vs;
+	} else if(strstr(spec, "win32-g++")) {
+		maketool_list = maketool_list_mingw;
+	} else {
+		maketool_list = maketool_list_common;
+	}
 	for(n = 0; maketool_list[n]; ++n)
 	{
 		if(qc_verbose)
@@ -648,6 +846,8 @@ static int do_conf(qcdata_t *q, const char *argv0)
 {
 	char *qmake_path;
 	char *maketool;
+	int qt_maj_version = 0;
+	char *specs_name = NULL;
 	int n;
 	int ret;
 
@@ -660,7 +860,7 @@ static int do_conf(qcdata_t *q, const char *argv0)
 			try_print_var(q->args[n].envvar, q->args[n].val);
 	}
 
-	printf("Verifying Qt 4 build environment ... ");
+	printf("Verifying Qt build environment ... ");
 	fflush(stdout);
 
 	if(qc_verbose)
@@ -674,7 +874,7 @@ static int do_conf(qcdata_t *q, const char *argv0)
 		else
 			printf("fail\n");
 		printf("\n");
-		printf("Reason: Unable to find the 'qmake' tool for Qt 4.\n");
+		printf("Reason: Unable to find the 'qmake' tool for Qt %s.\n", qtsearchtext);
 		printf("\n");
 		printf("%s", q->qtinfo);
 		return 0;
@@ -682,13 +882,75 @@ static int do_conf(qcdata_t *q, const char *argv0)
 	if(qc_verbose)
 		printf("qmake found in %s\n", qmake_path);
 
+	// figure out what version it is.
+	{
+		char buf[20]; // version string output always small
+		if (qmake_query_maj_ver(qmake_path, buf, sizeof(buf))) {
+			qt_maj_version = strtoul(buf, NULL, 10);
+			if (qt_maj_version == ULONG_MAX)
+				qt_maj_version = 0;
+		}
+		if (qt_maj_version == 0) {
+			if(qc_verbose) {
+				printf(" -> fail\n");
+				printf("Qt major version is not parsable\n");
+			} else
+				printf("fail\n");
+			free(qmake_path);
+			return 0;
+		}
+	}
+
+	// find specc name
+	{
+		char *spec = get_envvar("QMAKESPEC");
+		if (!spec) {
+			// try to find default spec
+			if (qt_maj_version > 4) {
+				char buf[32];
+				if (qmake_query(qmake_path, "QMAKE_XSPEC", buf, sizeof(buf))) // or QMAKE_SPEC ?
+					specs_name = strdup(buf);
+				// it's possible to compute full path like QT_HOST_DATA/QMAKE_XSPEC
+			} else {
+				char *qmake_conf, buf[1024];
+				FILE *fp;
+				qmake_query(qmake_path, "QMAKE_MKSPECS", buf, sizeof(buf));
+				qmake_conf = append_free(strdup(buf), "/default/qmake.conf");
+				fp = fopen(qmake_conf, "r");
+				free(qmake_conf);
+				if (fp) {
+					size_t br;
+					if ((br = fread(buf, 1, sizeof(buf) - 1, fp)) > 0) {
+						char *str, *end;
+						buf[br] = 0;
+						str = strstr(buf, "QMAKESPEC_ORIGINAL=");
+						if (str) {
+							str = extract_spec_name(str);
+							if (str) {
+								specs_name = strdup(str);
+							}
+						}
+					}
+					fclose(fp);
+				}
+			}
+		} else {
+			char *s = strdup(spec);
+			specs_name = extract_spec_name(s);
+			if (specs_name) {
+				specs_name = strdup(specs_name);
+			}
+			free(s);
+		}
+	}
+
 	// TODO: in verbose mode, print out default makespec and what we're
 	//   overriding the makespec to (if any).  since at this time we don't
 	//   ever override the makespec on windows, we don't need this yet.
 
 	cleanup_qconftemp();
 	maketool = NULL;
-	if(!do_conf_create(q, qmake_path, &maketool))
+	if(!do_conf_create(q, qmake_path, specs_name, &maketool))
 	{
 		cleanup_qconftemp();
 		if(qc_verbose)
@@ -816,6 +1078,31 @@ int main(int argc, char **argv)
 			qc_verbose = 1;
 			set_envvar("QC_VERBOSE", "Y");
 		}
+		else if(strcmp(var, "prefix") == 0)
+		{
+			prefix = strdup(val);
+		}
+		else if(strcmp(var, "bindir") == 0)
+		{
+			bindir = strdup(val);
+		}
+		else if(strcmp(var, "includedir") == 0)
+		{
+			includedir = strdup(val);
+		}
+		else if(strcmp(var, "datadir") == 0)
+		{
+			datadir = strdup(val);
+		}
+		else if(strcmp(var, "libdir") == 0)
+		{
+			libdir = strdup(val);
+		}
+		else if(strcmp(var, "qtselect") == 0)
+		{
+			if (val && strlen(val))
+				qc_qtselect = parse_qtselect(val);
+		}
 		else
 		{
 			at = find_arg(q->args, q->args_count, var);
@@ -856,14 +1143,82 @@ int main(int argc, char **argv)
 		qcdata_delete(q);
 		if(ex_qtdir)
 			free(ex_qtdir);
+
+		if(qc_qtselect)
+			free(qc_qtselect);
+
+		if(prefix)
+			free(prefix);
+
+		if(bindir)
+			free(bindir);
+
+		if(includedir)
+			free(includedir);
+
+		if(libdir)
+			free(libdir);
+
+		if(datadir)
+			free(datadir);
+		
 		return 1;
 	}
 
+	if(prefix)
+	{
+		set_envvar("PREFIX", prefix);
+		if(!libdir)
+			libdir = append_str(prefix, "/lib");
+
+		if(!bindir)
+			bindir = append_str(prefix, "/bin");
+
+		if(!includedir)
+			includedir = append_str(prefix, "/include");
+
+		if(!datadir)
+			datadir = append_str(prefix, "/share");
+	}
+
+	if(libdir)
+		set_envvar("LIBDIR", libdir);
+
+	if(bindir)
+		set_envvar("BINDIR", bindir);
+
+	if(includedir)
+		set_envvar("INCLUDEDIR", includedir);
+
+	if(datadir)
+		set_envvar("DATADIR", datadir);
+
+	if(libdir)
+		set_envvar("LIBDIR", libdir);
+
 	n = do_conf(q, argv[0]);
 	qcdata_delete(q);
 	if(ex_qtdir)
 		free(ex_qtdir);
 
+	if(qc_qtselect)
+		free(qc_qtselect);
+
+	if(prefix)
+		free(prefix);
+
+	if(bindir)
+		free(bindir);
+
+	if(includedir)
+		free(includedir);
+
+	if(libdir)
+		free(libdir);
+
+	if(datadir)
+		free(datadir);
+
 	if(n)
 		return 0;
 	else
