@@ -747,7 +747,7 @@ private:
 		"			shift\n"
 		"			;;\n"
 		"		--help) show_usage; exit ;;\n"
-		"		*) show_usage; exit 1;;\n"
+		"		*) echo \"configure: WARNING: unrecognized options: $1\" >&2; shift; ;;\n"
 		"	esac\n"
 		"done\n\n";
 
@@ -785,7 +785,7 @@ private:
 		"		fi\n"
 		"		vout=`/bin/sh -c \"$cmd\" 2>&1`\n"
 		"		case \"${vout}\" in\n"
-		"			?.?.?)\n"
+		"			*.*.*)\n"
 		"				vmaj=\"${vout%%.*}\"\n"
 		"				if [ ! -z \"$QC_QTSELECT\" ]; then\n"
 		"					if [ \"$vmaj\" = \"$QC_QTSELECT\" ]; then\n"
